* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
OPTIONS NOnumber NOdate NOcenter FormDlim=" " PageSize=MAX LineSize=MAX SPOOL;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; *ODS LISTING;

***********************************************************************************;
*******     MACRO PROGRAMS TO AUTOMATE CALCULATIONS FOR THIS EXAMPLE        *******;
*******               NOTHING IN HERE NEEDS TO BE CHANGED                   *******;
***********************************************************************************;

* To use TotalR2 macro;
* DV =        Case-sensitive name of dependent variable (required);
* PredFewer = Name of OUTPM= data file of predicted outcomes for first model (required);
* PredMore =  Name of OUTPM= data file of predicted outcomes for second model (optional);
%MACRO TotalR2(DV=, Pred1=, Pred2=);
PROC CORR NOPRINT NOSIMPLE DATA=&Pred1. OUTP=Corr1; VAR pred &DV.; RUN;
DATA Corr1; LENGTH Name $30.; SET Corr1; Name="&Pred1.";
     IF Pred=. THEN Pred=0; PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; 
     KEEP Name PredCorr TotalR2; RUN;
%IF %LENGTH(&Pred2.)>0 %THEN %DO;
PROC CORR NOPRINT NOSIMPLE DATA=&Pred2. OUTP=Corr2; VAR pred &DV.; RUN;
DATA Corr2; LENGTH Name $30.; SET Corr2; Name="&Pred2."; 
     IF Pred=. THEN Pred=0; PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; 
     KEEP Name PredCorr TotalR2; RUN;
DATA CorrCompare; LENGTH Name $30.; SET Corr1 Corr2; 
     TotalR2Diff=TotalR2-LAG1(TotalR2);
     KEEP Name PredCorr TotalR2 TotalR2Diff; RUN; 
%END;
%IF %LENGTH(&Pred2.)>0 %THEN %LET ToPrint=CorrCompare; 
     %ELSE %LET ToPrint=Corr1;
TITLE8 "Total-R2 Per Model";
%IF %LENGTH(&Pred2.)>0 %THEN %DO;
TITLE9 "Change in Total-R2 for &Pred1. vs. &Pred2."; %END;
PROC PRINT NOOBS DATA=&ToPrint.; RUN; TITLE8; TITLE9;   
%MEND TotalR2;

* To use PseudoR2 macro;
* Ncov =     TOTAL # entries in covariance parameter estimates table;
* CovFewer = Name of ODS CovParms table for nested baseline model (required);
* CovMore1 = Name of ODS CovParms table for comparison model1 (required);
* CovMore2 = Name of ODS CovParms table for comparison model2 (optional);
%MACRO PseudoR2(NCov=, CovFewer=, CovMore1=, CovMore2=);
DATA &CovFewer.; LENGTH Name $30.; SET &CovFewer.; Name="&CovFewer."; RUN;
DATA &CovMore1.; LENGTH Name $30.; SET &CovMore1.; Name="&CovMore1."; RUN;
DATA CovCompare1; LENGTH Name $30.; SET &CovFewer. &CovMore1.; RUN;
DATA CovCompare1; SET CovCompare1; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
%IF %LENGTH(&CovMore2.)>0 %THEN %DO; %LET lagchange=%EVAL(&Ncov.*2);
DATA &CovMore2.; LENGTH Name $30.; SET &CovMore2.; Name="&CovMore2."; RUN;
DATA CovCompare2; LENGTH Name $30.; SET &CovFewer. &CovMore2.; RUN;
DATA CovCompare2; SET CovCompare2; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
DATA CovCompare; SET CovCompare1 CovCompare2;
     PseudoR2Change=PseudoR2-(LAG&lagchange(PseudoR2)); RUN;
%END; %IF %LENGTH(&CovMore2.)>0 %THEN %LET ToPrint=CovCompare; 
     %ELSE %LET ToPrint=CovCompare1;
DATA &ToPrint.; SET &ToPrint.; 
     IF CovParm IN("UN(2,1)","UN(3,1)","UN(3,2)","UN(4,1)","UN(4,2)","UN(4,3)",
                   "UN(5,1)","UN(5,2)","UN(5,3)","UN(5,4)",
                   "UN(6,1)","UN(6,2)","UN(6,3)","UN(6,4)","UN(6,5)") THEN DELETE; 
	DROP Zvalue ProbZ; RUN;
TITLE8 "Pseudo-R2 Relative to &CovFewer.";
%IF %LENGTH(&CovMore2.)>0 %THEN %DO;
TITLE9 "Change in Pseudo-R2 for &CovMore1. vs. &CovMore2."; %END;
PROC PRINT NOOBS DATA=&ToPrint.; RUN; TITLE8; TITLE9;
%MEND PseudoR2;

***********************************************************************************;
*******           BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 3           *******;
*******               CHANGE "filesave" TO YOUR FOLDER PATH                 *******;
***********************************************************************************;

* Define global variable for file location to be replaced in code below;
* \\Client\ precedes actual path when using UIowa Virtual Desktop;
%LET filesave=C:\Dropbox\23_PSQF6272\PSQF6272_Example3;

* Import trimmed example data excel data file from sheet "HSB4" into SAS;
PROC IMPORT DATAFILE="&filesave.\Example3_Data.xlsx" OUT=work.Example3 DBMS=XLSX REPLACE; 
     SHEET="HSB4"; GETNAMES=YES; RUN;

* Open external file to save results to;
ODS RTF FILE="&filesave.\Example3_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;

* Filter to only cases complete on all variables to be used below (before cluster means);
DATA work.Example3; SET work.Example3; 
     IF NMISS(math,size,private)>0 THEN DELETE;
RUN;

* Compute cluster means for level-1 variables;
PROC SORT DATA=work.Example3; BY schoolID; RUN; 
PROC MEANS NOPRINT DATA=work.Example3; BY schoolID;
     VAR  math ses nonwhite; OUTPUT OUT=work.ClusterMeans
     MEAN(math ses nonwhite) = CMmath CMses CMnon; RUN;

TITLE "SAS Descriptive Statistics for Cluster Mean Level-2 Variables";
PROC MEANS NDEC=2  DATA=work.ClusterMeans; VAR CMmath CMses CMnon; RUN; TITLE;
PROC CORR NOSIMPLE DATA=work.ClusterMeans; VAR CMmath CMses CMnon; RUN; TITLE;

TITLE "SAS Descriptive Statistics for Original Level-1 Variables";
PROC MEANS NDEC=2  DATA=work.Example3; VAR math ses nonwhite; RUN; TITLE;

* Merge in cluster means and center predictors;
DATA work.Example3; MERGE work.Example3 work.ClusterMeans; BY schoolID;
* Center L2 predictor at constant so 0 is meaningful;
  CMnon30 = CMnon - .30; 
* 0 is otherwise already meaningful in other predictors;
* Center L1 predictors at cluster means;
  WCses = ses - CMses;
  WCnon = nonwhite - CMnon;
* Center outcome at cluster mean for correlations;
  WCmath = math - CMmath;
* Filter to only cases complete on all variables to be used below;
  IF NMISS(math,ses,nonwhite)>0 THEN DELETE;
RUN;

TITLE "SAS Descriptive Statistics for Within-Cluster Level-1 Variables";
PROC MEANS NDEC=2  DATA=work.Example3; VAR WCmath WCses WCnon; RUN; TITLE;
PROC CORR NOSIMPLE DATA=work.Example3; VAR WCmath WCses WCnon; RUN; TITLE;


***********************************************************************************;
*******                    BEGIN MODELS FOR EXAMPLE 3                       *******;
***********************************************************************************;

* Empty Models;

TITLE "SAS Model 1: Empty Means, Random Intercept for Math";
PROC MIXED DATA=work.Example3 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID; 
     MODEL math = / SOLUTION DDFM=Satterthwaite; 
     RANDOM INTERCEPT / VCORR TYPE=UN SUBJECT=schoolID; * VCORR gives ICC;
     ODS OUTPUT CovParms=CovEmpty; * Save variances; 
RUN; TITLE;

TITLE "SAS: Empty Means, Random Intercept for SES";
PROC MIXED DATA=work.Example3 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID; 
     MODEL ses = / SOLUTION DDFM=Satterthwaite; 
     RANDOM INTERCEPT / VCORR TYPE=UN SUBJECT=schoolID; * VCORR gives ICC;
RUN; TITLE;

TITLE "SAS: Empty Means, Random Intercept for Binary Nonwhite";
PROC GLIMMIX DATA=work.Example3 NOCLPRINT NAMELEN=100 METHOD=QUAD(QPOINTS=7);
     CLASS schoolID; 
     MODEL nonwhite (DESCENDING) = / SOLUTION LINK=LOGIT DIST=BINARY DDFM=NONE; 
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     COVTEST "Need Random Intercept?" ZEROG;
RUN; TITLE;


***********************************************************************************;
* Add SES predictors;

TITLE "SAS Model 2a: Add L1 Cluster-MC Student SES";
PROC MIXED DATA=work.Example3 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = WCses / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     ODS OUTPUT CovParms=CovSESwithin; * Save variances; 
RUN; TITLE;
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore1=CovSESWithin);

TITLE "SAS Model 2b: L1 Cluster-MC Student SES + L2 Cluster Mean SES";
PROC MIXED DATA=work.Example3 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = WCses CMses / SOLUTION DDFM=Satterthwaite OUTPM=PredSES;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     ESTIMATE "SES L2 Contextual Slope" WCses -1 CMses 1;
     ODS OUTPUT CovParms=CovSESbetween; * Save variances; 
RUN; TITLE;
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore1=CovSESwithin, CovMore2=CovSESbetween);

TITLE "SAS Model 3a: Use Smushed Main Effect of Student SES instead";
PROC MIXED DATA=work.Example3 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = ses / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     ODS OUTPUT CovParms=CovSESsmush; * Save variances; 
RUN; TITLE;
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore1=CovSESsmush);

TITLE "SAS Model 3b: L1 Grand-MC Student SES + L2 Cluster Mean SES";
PROC MIXED DATA=work.Example3 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = ses CMses / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     ESTIMATE "SES L2 Between Slope" ses 1 CMses 1;
     ODS OUTPUT CovParms=CovSEScontext; * Save variances; 
RUN; TITLE;
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore1=CovSESsmush, CovMore2=CovSEScontext);


***********************************************************************************;
* Add nonwhite predictors;

TITLE "SAS Model 4a: Add L1 Binary Student Nonwhite";
PROC MIXED DATA=work.Example3 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = ses CMses nonwhite / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     ODS OUTPUT CovParms=CovNWsmush; * Save variances; 
RUN; TITLE;
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore1=CovSESbetween, CovMore2=CovNWsmush);

TITLE "SAS Model 4b: L1 Binary Student Nonwhite + L2 Cluster Mean Nonwhite";
PROC MIXED DATA=work.Example3 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = ses CMses nonwhite CMnon30 / SOLUTION DDFM=Satterthwaite;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     ESTIMATE "SES L2 Between Slope"         ses 1 CMses 1;
     ESTIMATE "Nonwhite L2 Between Slope"    nonwhite 1 CMnon30 1;
     ODS OUTPUT CovParms=CovNWcontext; * Save variances; 
RUN; TITLE;
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore1=CovNWsmush, CovMore2=CovNWcontext);

TITLE "SAS Model 4c: Use Cluster-MC versions instead";
PROC MIXED DATA=work.Example3 NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID;         
     MODEL math = WCses CMses WCnon CMnon30 / SOLUTION DDFM=Satterthwaite OUTPM=PredSESnon;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID; 
     ESTIMATE "SES L2 Contextual Slope"      WCses -1 CMses 1;
     ESTIMATE "Nonwhite L2 Contextual Slope" WCnon -1 CMnon30 1;
     ESTIMATE "Nonwhite L2 Between Slope per 10%"             CMnon30 1 / DIVISOR=10;
     ESTIMATE "Nonwhite L2 Contextual Slope per 10%" WCnon -1 CMnon30 1 / DIVISOR=10;
     ODS OUTPUT CovParms=CovNWbetween; * Save variances; 
RUN; TITLE;
%PseudoR2(NCov=2, CovFewer=CovEmpty, CovMore1=CovSESbetween, CovMore2=CovNWbetween);
%TotalR2(DV=math, Pred1=PredSES, Pred2=PredSESnon);


***********************************************************************************;
* Close external file of saved results;
ODS RTF CLOSE;
