// Prevent "more" messages from appearing
   set more off
// Control line length
   set linesize 150

///////////////////////////////////////////////////////////////////////////////////////
////////               BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 3          ///////
////////                  CHANGE "filesave" TO YOUR FOLDER PATH                 ///////
///////////////////////////////////////////////////////////////////////////////////////

// Define global variable for file location to be replaced in code below
// \\Client\ precedes path in Virtual Desktop outside H drive
   global filesave "C:\Dropbox\23_PSQF6272\PSQF6272_Example3"

// Open trimmed example excel data file from sheet "HSB4" and clear away any existing data
   clear // clear memory in case of open data
   import excel "$filesave\Example3_Data.xlsx", firstrow case(preserve) sheet("HSB4") clear 
   
// Open external file to save results to
   log using $filesave\Example3_STATA_Output.log, replace   

// Filter to only cases complete on all variables to be used below (before cluster means)
   egen nmiss=rowmiss(math ses nonwhite)
   drop if nmiss>0
	  
// Compute cluster means for level-1 variables
   sort schoolID
   egen CMmath = mean(math),     by(schoolID)     
   egen CMses  = mean(ses),      by(schoolID)     
   egen CMnon  = mean(nonwhite), by(schoolID)     
     
display "STATA Descriptive Statistics for Cluster Mean Level-2 Variables"
summarize CMmath CMses CMnon
pwcorr    CMmath CMses CMnon, sig

display "STATA Descriptive Statistics for Original Level-1 Variables"
summarize math ses nonwhite

// Center L2 predictor at constant so 0 is meaningful
   gen CMnon30 = CMnon - .30
// 0 is otherwise already meaningful in other predictors

// Center L1 predictors at cluster means
   gen WCses = ses - CMses
   gen WCnon = nonwhite - CMnon
// Center outcome at cluster mean for correlations
   gen WCmath = math - CMmath

display "STATA Descriptive Statistics for Within-Cluster Level-1 Variables"
summarize WCmath WCses WCnon
pwcorr    WCmath WCses WCnon, sig
     

///////////////////////////////////////////////////////////////////////////////////////
////////                     BEGIN MODELS FOR EXAMPLE 3                          //////
///////////////////////////////////////////////////////////////////////////////////////

// Empty models

display "STATA Model 1: Empty Means, Random Intercept for Math"
mixed math , || schoolID: , reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat icc                       // Intraclass correlation

display "STATA: Empty Means, Random Intercept for SES"
mixed ses , || schoolID: , reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat icc                       // Intraclass correlation
  
display "STATA: Empty Means, Random Intercept for Binary Nonwhite"
melogit nonwhite , || schoolID: , nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat icc                       // Intraclass correlation  
          
// Add SES predictors               
                
display "STATA Model 2a: Add L1 Cluster-MC Student SES"
mixed math c.WCses, || schoolID: , reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  

display "STATA Model 2b: L1 Cluster-MC Student SES + L2 Cluster Mean SES"
mixed math c.WCses c.CMses, || schoolID: , ///
           reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
lincom c.WCses*-1 + c.CMses*1   // SES L2 Contextual Slope
predict predSES                 // Save fixed-effect predicted outcomes for total-R2
 
display "STATA Model 3a: Use Smushed Main Effect of L1 Student SES Instead"
mixed math c.ses, || schoolID: , reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  

display "STATA Model 3b: L1 Grand-MC Student SES + L2 Cluster Mean SES"
mixed math c.ses c.CMses, || schoolID: , ///
           reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
lincom c.ses*1 + c.CMses*1      // SES L2 Between Slope  

// Add nonwhite predictors    

display "STATA Model 4a: Add L1 Binary Student Nonwhite"
mixed math c.ses c.CMses c.nonwhite, || schoolID: , ///
           reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2         // Print -2LL for model   

display "STATA Model 4b: L1 Binary Student Nonwhite + L2 Cluster Mean Nonwhite"
mixed math c.ses c.CMses c.nonwhite c.CMnon30, || schoolID: , ///
           reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2         // Print -2LL for model  
lincom c.ses*1 + c.CMses*1         // SES L2 Between Slope  
lincom c.nonwhite*1 + c.CMnon30*1  // Nonwhite L2 Between Slope 

display "STATA Model 4c: Use Cluster-MC Versions of L1 Predictors Instead"
mixed math c.WCses c.CMses c.WCnon c.CMnon30, || schoolID: , ///
           reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2         // Print -2LL for model  
lincom c.WCses*-1 + c.CMses*1      // SES L2 Contextual Slope  
lincom c.WCnon*-1 + c.CMnon30*1    // Nonwhite L2 Contextual Slope 
lincom c.CMnon30*1/10                    // Nonwhite L2 Between Slope per 10% 
lincom c.WCnon*-1/10 + c.CMnon30*1/10    // Nonwhite L2 Contextual Slope per 10%
predict predSESnon              // Save fixed-effect predicted outcomes for total-R2      
corr math predSES               // Get total r to make R2
display "Total-R2 = " r(rho)^2  // Print total R2 for SES relative to empty model   
corr math predSESnon            // Get total r to make R2
display "Total-R2 = " r(rho)^2  // Print total R2 for SES+Nonwhite relative to empty model 


// Close external file of saved results
   log close

